@import url("resource:///org/gnome/builder/themes/shared/shared-layout.css");
@import url("resource:///org/gnome/builder/themes/shared/shared-editor.css");
@import url("resource:///org/gnome/builder/themes/shared/shared-search.css");
@import url("resource:///org/gnome/builder/themes/shared/shared-treeview.css");

/* work around some gtk padding issue */
filechooser actionbar button.combo {
  padding: 0;
}

/* Generic styles */
.warning {
    color: @warning_color;
}

/* styling for spellchecker */
.gb-spellchecker list button.close:disabled,
.gb-spellchecker list button.close {
  background: none;
  border: none;
  box-shadow: none;
  padding-left: 4px;
  padding-right: 4px;
}
.gb-spellchecker list row label {
  padding: 7px;
}
.gb-spellchecker list row {
  border-bottom: 1px solid alpha(@borders, 0.2);
}
.gb-spellchecker list row:last-child {
  border-bottom: none;
}
.countbox {
  background-color: #b3d4fc;
  border-radius: 3px;
  padding: 3px;
}



/*
 * For our pattern of popover with lists (and close buttons).
 */
popover list row {
  padding: 6px;
}
popover list row button:not(.circular) {
  background: transparent;
  border: none;
  box-shadow: none;
  margin: 0;
  padding: 0;
  opacity: 0.25;
}
popover list row:selected button:not(.circular),
popover list row:selected button:hover:not(.circular) {
  color: @theme_selected_fg_color;
}
popover list row button:hover:not(.circular) {
  opacity: 1;
}


/*
 * Styling in the genesis (create project) perspective.
 */
genesisperspective stack > box:first-child list row {
  padding: 10px;
  border-bottom: 1px solid alpha(@borders, 0.2);
}
genesisperspective stack > box:first-child list row:last-child {
  border-bottom: none;
}

/*
 * Greeter tweaks
 *
 * The following tweaks the greeter perspective by adding
 * separator lines to the list box.
 */
greeter list row {
  border-bottom: 1px solid alpha(@borders, 0.2);
}
greeter list row:last-child {
  border-bottom: none;
}
greeter frame border {
  border-color: alpha(@borders, 0.6);
}

/*
 * Perspectives switcher
 *
 * The following tweaks the left-most sidebar containing
 * the list of perspectives.
 */
perspectiveswitcher {
  border-right: 1px solid alpha(@borders, 0.5);
}
perspectiveswitcher button {
  background: transparent;
  border-radius: 0;
  border: none;
  box-shadow: none;
  padding: 6px;
}


/* Workaround Adwaita adding borders we don't want */
textview border.left {
  background: none;
}

treeview.dim-label {
  color: alpha(currentColor, 0.5);
}


button.run-arrow-button {
  min-width: 12px;
}


popover.popover-selector list {
  padding: 12px;
}

popover.popover-selector list row {
  padding: 8px;
}

popover.popover-selector list row image {
  min-height: 16px;
  min-width: 16px;
  margin-right: 12px;
}
popover.popover-selector list row image:dir(rtl) {
  margin-right: 0px;
  margin-left: 12px;
}
popover.popover-selector list row .accel {
  margin-left: 20px;
  margin-right: 0;
}
popover.popover-selector list row .accel:dir(rtl) {
  margin-left: 0;
  margin-right: 20px;
}


popover.omnibar list row {
  border-bottom: 1px solid alpha(@borders, 0.4);
}
popover.omnibar list row:last-child {
  border-bottom: none;
}


omnibar entry {
  background-color: @theme_bg_color;
  color: alpha(@theme_fg_color, 0.8);
}
omnibar:hover entry,
omnibar:active entry {
  background-color: mix(@theme_bg_color, @content_view_bg, 0.9);
  color: @theme_fg_color;
}


buildperspective list.sidebar row:selected button:hover {
  border-color: transparent;
  box-shadow: none;
  background: transparent;
  color: @theme_selected_fg_color;
  opacity: 1;
}
buildperspective list.sidebar row:selected button,
buildperspective list.sidebar row:selected button:active {
  opacity: 0.8;
}
buildperspective list.sidebar {
  border-right: 1px solid alpha(@borders, 0.55);
}


configurationview list row {
  padding: 10px;
  border-bottom: 1px solid alpha(@borders, 0.4);
}
configurationview list row:last-child {
  border-bottom: none;
}

configurationview list row entry {
  background: transparent;
  border: none;
  padding: 0;
  padding-left: 5px;
  border-radius: 3px;
  margin: -5px;
}

configurationview list row spinbutton entry {
  margin-left: 2px;
}

configurationview list row button {
  margin: -5px 0;
}


/* hrmm, we can use this to get row separators */
configurationview treeview {
  border-bottom: 1px solid alpha(@borders, 0.4);
}


popover.transfers list {
  background-color: transparent;
}
popover.transfers list row {
  border-top: 1px solid @borders;
}
popover.transfers list row:first-child {
  border-top: none;
}
popover.transfers list row > box {
  padding: 10px;
}

button.run-arrow-button {
  padding: 0px;
}
