@import url("resource:///org/gnome/builder/themes/shared.css");

idelayoutstackheader > button:last-child {
  margin-right: 3px;
}

entry.search-missing {
  background-color: #cc0000;
  color: white;
  text-shadow: none;
}

entry.search-missing > image {
  color: white;
}


/* tweak icons for treeviews */
treeview.image { color: alpha(currentColor, 0.8); }
treeview.image:selected { color: alpha(@theme_selected_fg_color, 0.9); }


popover.messagepopover.background {
  border-bottom-left-radius: 9px;
  border-bottom-right-radius: 9px;
  padding: 0;
}

popover.messagepopover .popover-action-area button {
  padding: 7px 14px;;
  border-radius: 0;
  border-left-style: solid;
  border-right-style: none;
  border-bottom-style: none;
}

popover.messagepopover .popover-action-area button:first-child {
  border-left-style: none;
  border-bottom-left-radius: 7px;
  -gtk-outline-bottom-left-radius: 5px;
}

popover.messagepopover .popover-action-area button:last-child {
  border-right-style: none;
  border-bottom-right-radius: 7px;
  -gtk-outline-bottom-right-radius: 5px;
}

popover.messagepopover .popover-content-area {
  margin: 24px;
}

/* buildui panel */
ideeditorsidebar notebook header {
  background: transparent;
}
ideeditorsidebar notebook header tab {
  padding: 0;
}

ideeditorproperties entry:last-child {
  border-radius: 0;
  border-right: none;
  border-left: none;
}
